# R script file
# Author: Lars Relund
# Description: Create logvrp porblem instances.

### Chunk: Set themes for lattice package
# --------------------------------------------------------------------------------
# plot using xyplot
library(lattice)

pstheme <- canonical.theme(color = FALSE) ## in-built B&W theme
pstheme$strip.background$col <- "transparent" ## change strip bg
pstheme$superpose.line$lty<-1:10
pstheme$superpose.line$col<-rep(c(1,8),each=4)
pstheme$superpose.line$lwd<-c(rep(1,8))
pstheme$superpose.symbol$pch<-c(1,3,6,0,5,16,17)

pdftheme <- canonical.theme()
pdftheme$strip.background$col <- "transparent" ## change strip bg
pdftheme$background$col <- "transparent" ## change bg
pdftheme$superpose.line$lty<-1:10
pdftheme$superpose.symbol$pch<-rep(c(1,4),each=4)

lattice.options(default.theme = pstheme) ## set as default
fontf<-"Times"

## Some parameters that can be used for printing to file
devi="pdf"
ext=".pdf"
thm="pstheme"
# A4 is 8.25 x 11.75 inch
width=8.25
height=11.75
trellis.device(devi,file=paste("yield",ext,sep=""),family=fontf,theme=thm,width=width,height=height)
# plot script here
dev.off()

options(stringsAsFactors=FALSE)
### --------------------------------------------------------------------------------



### CHUNK: Create station set from imported set.
# ---------------------------------------------------------------------------------------------
tmp<-read.csv("stations.csv")
tmp$Station.Name<-paste("Kunde",0:(dim(tmp)[1]-1))
tmp$Station.Name[1]<-"Depot"
tmp$Address<-paste('"',tmp$Address,'"',sep='')

write.table(tmp,file="logvrp_stations_1-64.csv", col.names=F, row.names=F, quote=F, sep=",", fileEncoding="utf8")
stations<-read.table("logvrp_stations.csv",header=F,sep=",")
### -------------------------------------------------------------------------------------------




### CHUNK: Generate orders
# ---------------------------------------------------------------------------------------------

# function for generating orders without time windows
generateOrders<-function(stations,				# stations data frame
						 minAmount,				# minimum number of units in order
						 maxAmount,				# maximum number of units in order
						 numberOfOrders=0.9,	# proportion of customers 
						 depot=1,				# which row is depot in the stations data frame
						 loading="00:05:00",	# time of loading cargo
						 unloading="00:05:00"	# time of unloading cargo
) {
	customers<-as.character(stations[,1])
	depotName<-customers[depot]
	customers<-customers[-depot]
	names<-c("order_name", "from_station_name", "to_station_name", "amount", "load_type", "loading_duration", "unloading_duration", "tw_min_pick", "tw_max_pick", "tw_min_delivery", "tw_max_delivery")
	numberOfOrders<-floor(numberOfOrders*length(customers))
	customers<-sample(customers, numberOfOrders, replace=FALSE) 
	amount<-sample(minAmount:maxAmount,numberOfOrders,replace=T)
	tmp<-data.frame(paste("Order",1:numberOfOrders),rep(depotName,numberOfOrders),customers,amount,1,loading,unloading,NA,NA,NA,NA)
	names(tmp)<-names
	return(tmp)
}

# generate orders for 4 days current customer set
prefix<-"1-64"
stations<-read.table(paste("logvrp_stations_",prefix,".csv",sep=""),header=F,sep=",")
orders<-generateOrders(stations,1,5)
write.table(orders,file=paste("logvrp_orders_no_tw_",prefix,"_day1.csv",sep=""), row.names=F, col.names=F, na="", quote=F, sep=",", fileEncoding="utf8")
orders<-generateOrders(stations,1,3,numberOfOrders=0.5)
write.table(orders,file=paste("logvrp_orders_no_tw_",prefix,"_day2.csv",sep=""), row.names=F, col.names=F, na="", quote=F, sep=",", fileEncoding="utf8")
orders<-generateOrders(stations,3,5)
write.table(orders,file=paste("logvrp_orders_no_tw_",prefix,"_day3.csv",sep=""), row.names=F, col.names=F, na="", quote=F, sep=",", fileEncoding="utf8")
orders<-generateOrders(stations,1,7,numberOfOrders=0.8)
write.table(orders,file=paste("logvrp_orders_no_tw_",prefix,"_day4.csv",sep=""), row.names=F, col.names=F, na="", quote=F, sep=",", fileEncoding="utf8")


generateTimeW<-function(orders,					# set of orders
						start=10,				# start hour of TW
						finish=16,				# finish hour of TW
						numberOfTW=0.2,			# proportion of orders having TW
						gap=30					# gap in delivery (minutes)
						){
	numberOfTW<-floor(numberOfTW*dim(orders)[1])
	orderSet<-sample(1:dim(orders)[1],numberOfTW, replace=F)
	start<-sample(start:finish,numberOfTW,replace=T)

	orders$tw_min_delivery<-
}
### -------------------------------------------------------------------------------------------




### CHUNK: Split stations in two groups and generate orders
# ---------------------------------------------------------------------------------------------
stations<-read.table("logvrp_stations_1-64.csv",header=F,sep=",")
private<-sample(1:dim(stations)[1], dim(stations)[1]/2, replace=FALSE) 
stationsF<-stations[-private,]
stationsP<-stations[private,]
write.table(stationsF,file="logvrp_stations_firms.csv", col.names=F, row.names=F, quote=F, sep=",", fileEncoding="utf8")
write.table(stationsP,file="logvrp_stations_private.csv", col.names=F, row.names=F, quote=F, sep=",", fileEncoding="utf8")

# split orders 
orders<-read.table("logvrp_orders_no_tw_1-64_day1.csv",header=F,sep=",")
ordersF<-orders[orders[,3] %in% stationsF[,1],]
ordersP<-orders[orders[,3] %in% stationsP[,1],]
write.table(ordersF,file=paste("logvrp_orders_no_tw_",prefix,"_day1_firms.csv",sep=""), row.names=F, col.names=F, na="", quote=F, sep=",", fileEncoding="utf8")
write.table(ordersP,file=paste("logvrp_orders_no_tw_",prefix,"_day1_private.csv",sep=""), row.names=F, col.names=F, na="", quote=F, sep=",", fileEncoding="utf8")

orders<-read.table("logvrp_orders_no_tw_1-64_day2.csv",header=F,sep=",")
ordersF<-orders[orders[,3] %in% stationsF[,1],]
ordersP<-orders[orders[,3] %in% stationsP[,1],]
write.table(ordersF,file=paste("logvrp_orders_no_tw_",prefix,"_day1_firms.csv",sep=""), row.names=F, col.names=F, na="", quote=F, sep=",", fileEncoding="utf8")
write.table(ordersP,file=paste("logvrp_orders_no_tw_",prefix,"_day1_private.csv",sep=""), row.names=F, col.names=F, na="", quote=F, sep=",", fileEncoding="utf8")

orders<-read.table("logvrp_orders_no_tw_1-64_day3.csv",header=F,sep=",")
ordersF<-orders[orders[,3] %in% stationsF[,1],]
ordersP<-orders[orders[,3] %in% stationsP[,1],]
write.table(ordersF,file=paste("logvrp_orders_no_tw_",prefix,"_day1_firms.csv",sep=""), row.names=F, col.names=F, na="", quote=F, sep=",", fileEncoding="utf8")
write.table(ordersP,file=paste("logvrp_orders_no_tw_",prefix,"_day1_private.csv",sep=""), row.names=F, col.names=F, na="", quote=F, sep=",", fileEncoding="utf8")

orders<-read.table("logvrp_orders_no_tw_1-64_day4.csv",header=F,sep=",")
ordersF<-orders[orders[,3] %in% stationsF[,1],]
ordersP<-orders[orders[,3] %in% stationsP[,1],]
write.table(ordersF,file=paste("logvrp_orders_no_tw_",prefix,"_day1_firms.csv",sep=""), row.names=F, col.names=F, na="", quote=F, sep=",", fileEncoding="utf8")
write.table(ordersP,file=paste("logvrp_orders_no_tw_",prefix,"_day1_private.csv",sep=""), row.names=F, col.names=F, na="", quote=F, sep=",", fileEncoding="utf8")
### -------------------------------------------------------------------------------------------




### CHUNK: Create fleet set from imported set.
# ---------------------------------------------------------------------------------------------

tmp<-read.csv("fleet.csv")

# create truck
# Vehicle Name,is active,Load Type,Capacity,Per Km Cost,Fix Cost,Average Speed,Distance Unit,Starting Position,Stopping Position,Departure Date
truck<-c("Truck 2","true",1,100,1.5,0,60,"Km","Depot","Depot","2011-11-13 07:00:00")
tmp<-rbind(tmp,truck)

write.table(tmp,file="logvrp_fleet.csv", col.names=F, row.names=F, quote=F, sep=",", fileEncoding="utf8")
### -------------------------------------------------------------------------------------------





### CHUNK: Chunk 5 (empty) - Description (shown in the tree view)
# Comments/description here (not shown in tree view) ...
# ---------------------------------------------------------------------------------------------

### -------------------------------------------------------------------------------------------



### CHUNK: Chunk 6 (empty) - Description (shown in the tree view)
# Comments/description here (not shown in tree view) ...
# ---------------------------------------------------------------------------------------------

### -------------------------------------------------------------------------------------------
